/**
 * TimeValue.java
 */
package UTEStatsViewer;

/**
 * A time value is stored as a long with a unit of microseconds.  This
 * class defines static methods to format and parse time values.
 */

public final class TimeValue {
   /** This class only defines static methods. */
   private TimeValue() {
   }

   // The values are in microseconds.
   private static final int fractionalDigits = 6;
   private static final int scaleFactor = 1000000;


   /**
    * Return the time equivalent of a double value.
    */
   public static long from(double d) {
      double whole = (d >= 0) ? Math.floor(d) : Math.ceil(d);
      long fract = Math.round((d - whole) * scaleFactor);

      return ((long)whole * scaleFactor) + fract; 
   }


   /**
    * Return the internal value of a printed time value.
    */
   public static long parseTime(StringBuffer s) throws NumberFormatException {
      long accum = 0;
      int fractDigitCnt = 0;
      boolean hadDecimalPoint = false;
      boolean isNeg = false;
      int len = s.length();
      char c;

      try {
         int i = 0;
         c = s.charAt(i++);

         // skip initial whitespace
         while (c <= 0x20) {
            c = s.charAt(i++);
         }

         // check for leading minus sign
         if (c == '-') {
            isNeg = true;
            c = s.charAt(i++);
         }

         // accumulate value as negative of value so MIN_VALUE is
         // correctly handled
         do {
            if ( (c <= 0x20) || (fractDigitCnt >= fractionalDigits) ) {
               break;   // stop at whitespace or too many digits
            } else if (c == '.') {
               hadDecimalPoint = true;
            } else if ( (c >= '0') && (c <= '9') ) {
               accum = (accum * 10) - (c - '0');
               if (hadDecimalPoint) {
                  fractDigitCnt++;
               }
            } else {
               throw new NumberFormatException(s.toString());
            }
            if (i < len) {
               c = s.charAt(i++);
            } else {
              break;   // no more characters
            }
         } while (true);

         // fill in trailing fractional digits of zero that are missing
         for (; fractDigitCnt < fractionalDigits; fractDigitCnt++) {
            accum *= 10;
         }

         if (!isNeg) {
            accum = -accum;
         }
      } catch(StringIndexOutOfBoundsException ex) {
           throw new NumberFormatException(s.toString());
      }
      return accum;
   }


   /**
    * Return the double equivalent of a time value.
    */
   public static double todouble(long value) {
      return (value / scaleFactor) +
         Math.abs((double)(value % scaleFactor));
   }


   /**
    * Return the printable value of a time.
    */
   public static String toString(long value) {
      StringBuffer valueBuffer = new StringBuffer(21);
         // 19 digits + optional minus sign + optional decimal point
      long todo = value;
      boolean isNegative = (value < 0);
      boolean suppress = true;

      // format fraction digits
      for (int i = 0; i < fractionalDigits; i++) {
         int digit = Math.abs((int)(todo % 10));
         todo /= 10;
         if (digit != 0) {
            suppress = false;
         }
         if (!suppress) {
            valueBuffer.append((char)('0' + digit));
         }
      }

      // append a decimal point if there was a non-zero fraction
      if (!suppress) {
         valueBuffer.append('.');
      }

      // format remaining digits
      if (todo == 0) {
         valueBuffer.append("0");
      } else { 
         while (todo != 0) {
            int digit = Math.abs((int)(todo % 10));
            todo /= 10;
            valueBuffer.append((char)('0' + digit));
         }
      }
      if (value < 0) {
        valueBuffer.append('-');
      }

      // reverse the characters in the buffer
      int len = valueBuffer.length();
      int mid = len/2;
      for (int i = 0; i < mid; i++) {
         char c = valueBuffer.charAt(len - i - 1);
         valueBuffer.setCharAt(len - i - 1, valueBuffer.charAt(i));
         valueBuffer.setCharAt(i, c);
      }
      return valueBuffer.toString();
   }
}
