/*
 * IncrementalChoice.java
 */
package UTEStatsViewer;
import java.awt.*;
import java.awt.event.*;
/**
 * A Choice that contains a potentially large array of items.  A Choice
 * that contains a large number of items may take too much time to
 * construct to be useful in a GUI.  To reduce the amount of time it takes
 * to construct such a Choice, instead of constructing a Choice that 
 * contains all the items, a component containing the first few items is
 * constructed.  This component includes Buttons so that a user can add
 * items from the array.
 */
final class IncrementalChoice extends Container
       implements ActionListener, ItemListener {
   /** Default maximum size of native Choice. */
   public static final int CHOICE_SIZE_MAX;

   /** Default initial number of items in an IncrementalChoice */
   public static final int SIZE_INIT;

   /** Default number of items added to an IncrementalChoice */
   public static final int SIZE_INCR;

   static {
      // Set the default sizes.  These values should be computed at
      // run-time instead of being compile-time constants.  The
      // compile-time values are reasonable for a reasonable fast
      // machine.
      CHOICE_SIZE_MAX = 50;
      SIZE_INIT = 500;
      SIZE_INCR = 500;
   }

   /** Items to be in the list. */
   private Named items[];

   /** Choice used for normal sized array of items. */
   private Choice choice;

   /** Index of selected item. */
   private int selectedIndex;

   /** TextField used for big choice. */
   private TextField choiceField;

   /** Button used for big choice to popup list dialog. */
   private Button choiceButton;

   /** Number of items to be added to list when moreButton is pressed. */
   private int listSizeIncr;

   /** List of items for big choice. */
   private List choiceList;

   /** Index of first item in choiceList. */
   private int choiceListIndexFirst;

   /** Index of last item in choiceList. */
   private int choiceListIndexLast;

   /** Button used to add more items before the current first item. */
   private Button addToFrontButton;

   /** Button used to add more items after the current last item. */
   private Button addToRearButton;

   /** Dialog used to make selection from list. */
   private Dialog choiceListDialog;

   /** Label indicating how many items are shown in choiceList. */
   private Label sizeLabel;


   /** Hide default constructor. */
   private IncrementalChoice() {}


   /**
    * Construct an IncrementalChoice.
    * @param items  the items to be in the list
    * @param selectedIndex  index of initially selected item
    * @param choiceSizeMax  the maximum number of items to be handled
    *    by Choice
    * @param sizeInit  the initial number of items in the list
    * @param sizeIncr  the number of items to be added to the list at a time
    */
   IncrementalChoice(Named items[], int selectedIndex, int choiceSizeMax,
         int listSizeInit, int listSizeIncr)
   {
      // save copy of needed arguments
      this.items = items;
      this.listSizeIncr = listSizeIncr;
      this.selectedIndex = selectedIndex;

      setLayout(new BorderLayout());
      int rows = items.length;

      if (rows <= choiceSizeMax) {
         // use Choice
         choice = new Choice();
         for (int i = 0; i < rows; i++) {
            choice.add(items[i].getName());
         }
         choice.select(selectedIndex);
         add(choice, BorderLayout.CENTER);
      } else {
         // use choice field and a button to bring up list
         choiceField = new TextField(items[selectedIndex].getName());
         choiceField.setEditable(false);
         add(choiceField, BorderLayout.CENTER);
         choiceButton = new Button(Message.get(Message.select));
         choiceButton.addActionListener(this);
         add(choiceButton, BorderLayout.EAST);

         // construct initial list of items
         choiceList = new List(rows, false);
         choiceListIndexFirst = selectedIndex - (listSizeInit/2) - 1;
         choiceListIndexFirst = Math.max(choiceListIndexFirst, 0);
         choiceListIndexLast = choiceListIndexFirst + listSizeInit - 1;
         choiceListIndexLast = Math.min(choiceListIndexLast, (rows - 1));
         for (int i = choiceListIndexFirst; i <= choiceListIndexLast; i++) {
            choiceList.add(items[i].getName());
         }
         choiceList.select(selectedIndex - choiceListIndexFirst);

         // construct initial add items buttons
         addToFrontButton = new Button("+" +
               Math.min(listSizeIncr, choiceListIndexFirst));
         addToFrontButton.addActionListener(this);
         addToRearButton = new Button("+" +
               Math.min(listSizeIncr, rows - choiceListIndexLast - 1));
         addToRearButton.addActionListener(this);
         
      }
   }


   /**
    * Construct an IncrementalChoice with default size values.
    * @param items  the items to be in the list
    */
   IncrementalChoice(Named items[], int selectedIndex) {
      this(items, selectedIndex,
            CHOICE_SIZE_MAX, SIZE_INIT, SIZE_INCR);
   }


   /**
    * Return the index of the selected item.
    */
   public int getSelectedIndex() {
      return (choice != null) ? choice.getSelectedIndex() : selectedIndex;
   }


   // Implement ActionListener Interface
   public void actionPerformed(ActionEvent e) {
      Object source = e.getSource();
      if (source == choiceButton) {
         // determine parent to use for dialog
         Frame parentFrame = null;
         Container parent = this.getParent();
         while (! (parent instanceof Frame) ) {
            parent = parent.getParent();
         }

         // construct the dialog
         choiceListDialog = new Dialog((Frame)parent, true);
         /*choiceListDialog.addWindowListener(
	   DisposeOnClosing.getInstance());*/ //abhi's modification
         choiceListDialog.setLayout(new GridBagLayout());
         GridBagConstraints gbc = new GridBagConstraints();

         // add list to dialog
         gbc.gridx = gbc.gridy = 0;
         gbc.gridwidth = 1; gbc.gridheight = 2;
         gbc.fill = GridBagConstraints.BOTH;
         gbc.weightx = gbc.weighty = 1.0;
         choiceListDialog.add(choiceList, gbc);
         choiceList.makeVisible(choiceList.getSelectedIndex());

         // add the addToFrontButton
         gbc.gridx = 1; gbc.gridy = 0;
         gbc.gridwidth = gbc.gridheight = 1;
         gbc.fill = GridBagConstraints.NONE;
         gbc.anchor = GridBagConstraints.NORTH;
         gbc.weightx = gbc.weighty = 0.0;
         int toFrontCount = Math.min(listSizeIncr, choiceListIndexFirst);
         addToFrontButton.setLabel("+" + toFrontCount);
         addToFrontButton.setEnabled(toFrontCount > 0);
         choiceListDialog.add(addToFrontButton, gbc);

         // add the addToRearButton
         gbc.gridx = 1; gbc.gridy = 1;
         gbc.gridwidth = gbc.gridheight = 1;
         gbc.fill = GridBagConstraints.NONE;;
         gbc.anchor = GridBagConstraints.SOUTH;
         gbc.weightx = gbc.weighty = 0.0;
         int toRearCount = Math.min(listSizeIncr,
               items.length - choiceListIndexLast - 1);
         addToRearButton.setLabel("+" + toRearCount);
         addToRearButton.setEnabled(toRearCount > 0);
         choiceListDialog.add(addToRearButton, gbc);

         // add sizeLabel
         gbc.gridx = 0; gbc.gridy = 2;
         gbc.gridwidth = 2; gbc.gridheight = 1;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.weightx = 1.0; gbc.weighty = 0.0;
         sizeLabel = new Label(
               (choiceListIndexLast - choiceListIndexFirst + 1) +
               "/" + items.length, Label.CENTER);
         choiceListDialog.add(sizeLabel, gbc);

         // make size of dialog reasonable
         choiceListDialog.pack();
         Dimension windowSize = choiceListDialog.getSize();
         Dimension screenSize = choiceListDialog.getToolkit().getScreenSize();
         if (windowSize.height >= screenSize.height) {
            choiceListDialog.setSize(
                  windowSize.width + 30 /* add room for scrollbar */,
                  screenSize.height);
            choiceListDialog.validate();
         }
 
         // show the dialog and extract the results
         choiceList.addItemListener(this);
         choiceListDialog.show();
         choiceList.removeItemListener(this);
         selectedIndex = choiceList.getSelectedIndex() + choiceListIndexFirst;
         choiceField.setText(items[selectedIndex].getName());

      } else if (source == addToFrontButton) {
         // add items to front of list
         int listCount = choiceList.getItemCount();
         int choiceListIndexNew =
               Math.max(0, choiceListIndexFirst - listSizeIncr);
         for (int i = choiceListIndexFirst - 1; i >= choiceListIndexNew; i--) {
            choiceList.addItem(items[i].getName(), 0);
         }
         choiceListIndexFirst = choiceListIndexNew;

         // update button
         int remainCount = Math.min(listSizeIncr, choiceListIndexFirst);
         addToFrontButton.setLabel("+" + remainCount);
         addToFrontButton.setEnabled(remainCount > 0);

         // make selected item visible (in case adding items moved it)
         choiceList.makeVisible(choiceList.getSelectedIndex());

         sizeLabel.setText((choiceListIndexLast - choiceListIndexFirst + 1) +
               "/" + items.length);
         choiceListDialog.validate();

      } else if (source == addToRearButton) {
         // add items to rear of list
         int listCount = choiceList.getItemCount();
         int choiceListIndexNew =
               Math.min(items.length - 1, choiceListIndexLast + listSizeIncr);
         for (int i = choiceListIndexLast + 1; i <= choiceListIndexNew; i++) {
            choiceList.addItem(items[i].getName());
         }
         choiceListIndexLast = choiceListIndexNew;

         // update button
         int remainCount = Math.min(listSizeIncr,
              items.length -  choiceListIndexLast - 1);
         addToRearButton.setLabel("+" + remainCount);
         addToRearButton.setEnabled(remainCount > 0);

         // make selected item visible (in case adding items moved it)
         choiceList.makeVisible(choiceList.getSelectedIndex());

         sizeLabel.setText((choiceListIndexLast - choiceListIndexFirst + 1) +
               "/" + items.length);
         choiceListDialog.validate();

      }  // else ignore event
   }


   // Implement ItemListener interface
   public void itemStateChanged(ItemEvent e) {
      if ( (e.getItemSelectable() == choiceList) &&
           (e.getStateChange() == ItemEvent.SELECTED) ) {
         // dispose of modal dialog when item is selected
         if (choiceListDialog != null) {
            choiceListDialog.dispose();
            choiceListDialog = null;
         }
      }  // else ignore event
   }
}
